/*
 * Copyright (c) 2012 The University of Waikato.
 * Author: Joe Stringer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef CRC32C_H
#define CRC32C_H 1

#include "openvswitch/types.h"

ovs_be32 crc32c(const uint8_t *data, size_t);

#endif /* crc32c.h */
